/*
Jed Margolin 7/8/2024
Test for ADC and ds3231 Clock
Uses rpi-pico-v6 board
   LCD
   ADC
   DS3231 Real Time Clock (RTC)
   uart - 115200 baud
   nrf905 not used
*/

#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "hardware/adc.h"
#include "hardware/spi.h"
#include "main.h"
#include "hardware/uart.h"
#include <string.h>
#include <hardware/flash.h>
#include <hardware/sync.h>

#define QUIET 1       // 1 = Switching Voltage Regulator quiet mode

#define UART_TEST 0  // 1 = Test the uart by sending out time stamps, does not come back 
#define UART_ECHO 0  // 1 = Echo the keyboard, does not come back
//================================================================
bool repeating_timer_callback(struct repeating_timer *t)   // 4 ms 
{
   irq_flag = 1;

   // read switches
   do_switch(PB1, &switch1);   // PB1, switch structure  INC
   do_switch(PB2, &switch2);   // PB2, switch structure  DEC
   do_switch(PB3, &switch3);   // PB3, switch structure  MODE

   irq_count--;
   if(irq_count == 0)
   {
      irq_count = 250;   // 250 = 1 second
      secs_flag = 1;

      if(++frame == 3) 
      { 
         frame = 0;
         frame_flag = 1;  // every 3 seconds
      }
   }

   return (true);
}
//===============================================================
int main () 
{
   uint16_t n;

   nflag = 0;  // for flashing the LED
   frame = 0;
   frame_flag = 0;

   gpio_init(LED1);    // Green
   gpio_set_dir(LED1, GPIO_OUT);
   gpio_put(LED1,0);

   gpio_init(LED2);    // Red
   gpio_set_dir(LED2, GPIO_OUT);
   gpio_put(LED2,0);

   gpio_init(REGMODE);   // Regulator Switch Mode 1 = quiet
   gpio_set_dir(REGMODE, GPIO_OUT);
   gpio_put(REGMODE, QUIET);

   gpio_init(PB1);     // Pushbutton 1
   gpio_set_dir(PB1, GPIO_IN);
   gpio_pull_up(PB1);

   gpio_init(PB2);     // Pushbutton 2
   gpio_set_dir(PB2, GPIO_IN);
   gpio_pull_up(PB2);

   gpio_init(PB3);     // Pushbutton 3
   gpio_set_dir(PB3, GPIO_IN);
   gpio_pull_up(PB3);

   gpio_init(SW);     // Slide Switch for Mode (RX or Clock)
   gpio_set_dir(SW, GPIO_IN);
   gpio_pull_up(SW);

   gpio_init(CHANNEL);     // slide swutch selecting the nrf905 Channel
   gpio_set_dir(CHANNEL, GPIO_IN);
   gpio_pull_up(CHANNEL);
//-------------------
// Set up ADC
   adc_init();
   adc_gpio_init(ADC0_PIN);    // Make sure GPIO is high-impedance, no pullups etc
   adc_select_input(0);  // Select ADC input 0 (GPIO-26, pin 31)

   for(n=0; n<1024; n++)  { adc_buffer[n] = 0; }   // clear the buffer
//-------------------
// ds3231 uses I2C0 on the default SDA and SCL pins (4, 5)
   i2c_init(i2c_default, 300 * 1000);   // 300 KHz (400 KHz nax)
   gpio_set_function(PICO_DEFAULT_I2C_SDA_PIN, GPIO_FUNC_I2C);
   gpio_set_function(PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C);
   gpio_pull_up(PICO_DEFAULT_I2C_SDA_PIN);
   gpio_pull_up(PICO_DEFAULT_I2C_SCL_PIN);

// Make the I2C pins available to picotool
//    bi_decl(bi_2pins_with_func(PICO_DEFAULT_I2C_SDA_PIN, PICO_DEFAULT_I2C_SCL_PIN, GPIO_FUNC_I2C));
//-------------------
// nrf905 uses SPI-0 (but not used here)
// Enable SPI-0 at 5 MHz and connect to GPIOs
   spi_init(spi_default, 5000 * 1000);
   gpio_set_function(PICO_DEFAULT_SPI_RX_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_SCK_PIN, GPIO_FUNC_SPI);
   gpio_set_function(PICO_DEFAULT_SPI_TX_PIN, GPIO_FUNC_SPI);

   gpio_init(TXEN);    
   gpio_set_dir(TXEN, GPIO_OUT);
   gpio_put(TXEN,0);   // nrf905 Transmit Enable = High

   gpio_init(TRX_CE);    
   gpio_set_dir(TRX_CE, GPIO_OUT);
   gpio_put(TRX_CE,1);   // nrf905 RX Mode
   sleep_us(1000);

   gpio_init(DR);    
   gpio_set_dir(DR, GPIO_IN);
//   gpio_pull_up(DR);   // default is pull down

   gpio_init(AM);   // address match    
   gpio_set_dir(AM, GPIO_IN);
//   gpio_pull_up(AM);  // default is pull down

   gpio_init(CSN905);   // nrf905 CSN
   gpio_set_dir(CSN905, GPIO_OUT);
   gpio_put(CSN905,1);   

// nrd905 RX mode
   gpio_put(TXEN,0);
   sleep_us(20);
   gpio_put(TRX_CE,1);

// Make the SPI pins available to picotool
//   bi_decl(bi_4pins_with_func(PICO_DEFAULT_SPI_RX_PIN, PICO_DEFAULT_SPI_TX_PIN, PICO_DEFAULT_SPI_SCK_PIN, PICO_DEFAULT_SPI_CSN_PIN, GPIO_FUNC_SPI));

// CLK idle state = low, data sampled on rising edge and shifted on falling edge
// Does not support LSB first; fortunately the nrf905 is MSB first  
   spi_set_format(spi_default, 8, 0, 0, SPI_MSB_FIRST);  
//-------------------
// Wiggle the LEDs
   for(n=0; n<4; n++)   
   {
      gpio_put(LED2,nflag);   // Red LED
      nflag ^=1;
      gpio_put(LED1,nflag);   // Green LED
      sleep_ms(300);
   }
   gpio_put(LED1,0);
   gpio_put(LED2,0);
//------------------------------------
// Set up the UART with the required speed.
   uart_init(UART_ID, BAUD_RATE);

// Set the TX and RX pins by using the function select on the GPIO
   gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
   gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);

// Send the title message and version out through uart
   strcpy(line, version);
   strcat(line, "\n");

   log2_time_uart();          // Current time and date
   strcat(line,"\n\n");
   uart_puts(UART_ID, line);   // send it out

   sleep_ms(1000);
//----------------------------
   lcd_init();
//================================================
// Set some things
   mode = gpio_get(SW);   // adc test or ds3231 clock
   irq_flag = 0;
   irq_count = 250;
   secs_flag = 0;
   ts_number = 0;
   test_mode = 0;

// Greetings to LCD  - Canned Messages in Flash
   domsg(&tmsg1);    // "Jed Margolin's"
   domsg(&tmsg2);    // "ADC/RTC Test"

   sleep_ms(2000);   // 2 seconds

   SetCmdMode();     // set lcd for commands
   lcd_byte(0x01);   // clear screen

   domsg(&tmsg11);   // program version date
   
   sleep_ms(1000); 

   SetCmdMode();    // set lcd for commands
   lcd_byte(0x01);  // clear screen

   sleep_ms(1000);      // 1 second
//-------------------------------------
#if UART_TEST
   test_uart();  // Test the uart by sending out indexed Time Stamps, does not come back
#endif

#if UART_ECHO
   keyboard_echo();
#endif
//-------------------------------------
   SetCmdMode();        // set for commands
   lcd_byte(0x01);      // clear lcd, both lines

   domsg(&tmsg12);    // "nrf905"

   sleep_ms(1000);      // 1 second
   clear_lcd(0x80);     // clear lcd line 1 (address starts at 0x80)
   nrf905_init();       // Receive mode but not used here
//-------------------------------------
// Create a repeating timer that calls repeating_timer_callback
// If the delay is negative the next callback will be starting from the beginning of the callback
// Positive starts from the end of the callback
   add_repeating_timer_ms(-4,repeating_timer_callback, NULL, &timer);  // 4 ms

   if(mode)   { do_adc_tests(); }   // ADC Tests
   else       { do_clock(); }       // Display and set the ds3231; doesn't come back

} // end main
//======================================================================
void do_adc_tests()
{
   uint16_t n;

// make sure uart FIFO is empty, disable FIFO for this
   uart_set_fifo_enabled(UART_ID, false);
   for(n=0; n<32; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID) )   { uart_char = uart_getc(UART_ID); }
       else {break;}
   }
   //uart_set_fifo_enabled(UART_ID, true);  // Keep it off

// Send the title message and version out through uart
//   strcpy(line, version);
//   strcat(line, "\n");
//   uart_puts(UART_ID, line);   // send it out

   uart_puts(UART_ID, "Select the test by cycling through them using PB1 and PB2\n");

   sleep_ms(4000);

   test_mode = 0;

// secs_flag is every 1 second
   while(1)
   {
      while(!secs_flag);
      secs_flag = 0;

      if(switch1.press)
      {
         switch1.press = 0;
         test_mode++;
         if(test_mode == 7) { test_mode = 0;}
      }

      if(switch2.press)
      {
         switch2.press = 0;
         test_mode--;
         if(test_mode <0) { test_mode = 6;}
      }

      switch (test_mode)
      {
         case 0:
            if(frame_flag)   // every 3 seconds
            {
               frame_flag =  0;
               do_adc_test1(256,8);  // 256 samples = 8 bits
            }
            break;
        
         case 1:
           if(frame_flag)  // every 3 seconds
           {
              frame_flag = 0;
              do_adc_test1(32,5);     // 32 samples = 5 bits
           }
           break;

         case 2:
           if(frame_flag)  // every 3 seconds
           {
              frame_flag = 0;
              do_adc_test1(16,4);     // 16 samples = 4 bits
           }
           break;

         case 3:
           if(frame_flag)  // every 3 seconds
           {
              frame_flag = 0;
              do_adc_test1(8,3);     // 8 samples = 3 bits
           }
           break;

         case 4:
           if(frame_flag)  // every 3 seconds
           {
              frame_flag = 0;
              do_adc_test1(4,2);     // 4 samples = 2 bits
           }
           break;

         case 5:
           if(frame_flag)  // every 3 seconds
           {
              frame_flag = 0;
              do_adc_test1(2,1);     // 2 samples = 1 bit
           }
           break;

         case 6:
           frame_flag = 0;
           do_adc_test2();     // 1 sample
           break;

         default:
           break;
      }

   }  // end while

}  // end do_adc_tests
//====================================================
void do_adc_test1(uint16_t length, uint16_t length_bits)
{
   uint16_t m, n;

   SetCmdMode();      // set lcd for commands
   lcd_byte(0x01);    // clear screen
   domsg(&tmsg7);  // "Hi:"
   domsg(&tmsg8);  // "Lo:"
   domsg(&tmsg9);  // "D:"
   domsg(&tmsg10);  // "Avg:"

   // length samples, take high, low, difference, and average
   // 2.62 us/sample

      gpio_put(LED1,1);   
      for(n=0; n<length; n++)   
      {
         uint16_t result = adc_read();   // 2.5 us
         adc_buffer[n] = result;
      }
      gpio_put(LED1,0);    

      adc_sum = 0;
      adc_avg = 0;
      adc_high = 0;
      adc_low = 4095;

      for(n=0; n<length; n++)
      {
         adc_sum = adc_sum + adc_buffer[n];
         if(adc_buffer[n] < adc_low)  { adc_low = adc_buffer[n];}
         if(adc_buffer[n] > adc_high)  { adc_high = adc_buffer[n];}
      }

      adc_avg = adc_sum>>length_bits;
      adc_diff = adc_high - adc_low;

      uint16_bcd4_lcd(adc_high, 0x83);
      uint16_bcd4_lcd(adc_low, 0x8C);
      uint16_bcd4_lcd(adc_diff, 0xC2);
      uint16_bcd4_lcd(adc_avg, 0xCC);

      results_uart(length);  // send it out
//      sleep_ms(3000);

// Send all the values out to the uart
      strcpy(line,0);  // NULL string to start
      m = 0;

      if(length<16)
      {
         for(n=0; n<length; n++)
         {            
             my_itoa(adc_buffer[n], nstr);
             strcat(line, nstr);
             strcat(line,"   ");
         }
         strcat(line,"\n");
         uart_puts(UART_ID,line);
      }
      else  // greater than 16 
      {
         for(n=0; n<length; n++)
         {
            my_itoa(adc_buffer[n], nstr);
            strcat(line, nstr);
            strcat(line,"   ");
            if(m++ >14)
            {
               m = 0;
               strcat(line,"\n");
               uart_puts(UART_ID,line);  // do a line of 16 values
               strcpy(line,0);  // NULL string
            }
         }
      }  // end of length >16 values

      uart_puts(UART_ID,"\n");
/*
// Send all the values out to the uart
      strcpy(line,0);  // NULL string
      m = 0 ;
      for(n=0; n<length; n++)
      {
         my_itoa(adc_buffer[n], nstr);
         strcat(line, nstr);
         strcat(line,"   ");
         if(m++ >14)
         {
            m = 0;  // start the count over for the next line
            strcat(line,"\n");
            uart_puts(UART_ID,line);
            strcpy(line,0);  // NULL string
         }
      }

      uart_puts(UART_ID,"\n");
*/

      if(length >32) 
      { 
         results_uart(length);  // send it out again
      }
      else { uart_puts(UART_ID,"\n\n"); } 

}  // end do_adc_test1()
//===============================================
void results_uart(uint16_t samples)
{
   strcpy(line,"ADC High: ");
   my_itoa(adc_high,nstr);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line,"ADC Low: ");
   my_itoa(adc_low,nstr);
   strcat(line,nstr);
   strcat(line,"    ");
     
   strcat(line,"Difference: ");
   my_itoa(adc_diff,nstr);
   strcat(line,nstr);
   strcat(line,"    ");

   strcat(line,"Average: ");
   my_itoa(adc_avg,nstr);
   strcat(line,nstr);

   strcat(line,"   Samples: ");
   my_itoa(samples,nstr);
   strcat(line,nstr);
   strcat(line,"\n\n");

   uart_puts(UART_ID,line);
}
//======================================================================
void do_adc_test2()
{
   SetCmdMode();        // set for commands
   lcd_byte(0x01);      // clear lcd, both lines

   // Just do 1 read xx seconds. Display the value in hex, BCD, and binary
   uint16_t result = adc_read();   // 2.5 us

   uint16_hex_lcd(result, 0x80);    // unsigned int16 to BCD to lcd
   uint16_bcd4_lcd(result, 0x89);
   bin12_lcd(result, 0xC0);

// Send out to uart
   strcpy(line, "One sample per second    ");

   uint16_bcd_str(result, nstr);   // unsigned int16 to BCD string
   strcat(line, nstr);
   strcat(line,"    ");

   uint16_hex_str(result, nstr);    //unsigned int to hex string
   strcat(line, nstr);

   strcat(line,"    ");

   bin12_str(result, nstr);
   strcat(line, nstr);

   strcat(line,"\n");

   uart_puts(UART_ID,line);

   if(nflag)  
   { 
      gpio_put(LED1,1);
      gpio_put(LED2,0);
   }
   else       
   { 
      gpio_put(LED1,0);
      gpio_put(LED2,1);
   }
      
    nflag ^= 0x01;
   
}  // end do_adc_test2()
//===================================================
//===================================================
// wait for sw1 to be pressed
void wait_pb1()
{
   while(1)
   {
      if(switch1.press)
      {
         switch1.press = 0;   // reset the flag
         SetCmdMode();        // set lcd for commands
         lcd_byte(0x01);      // clear screen
         break;
     }
   }
}
//===================================================
void nrf905_init()
{
   unsigned char n, lcd_pos;

// Writing the Configuration and TXADR address
   if( gpio_get(CHANNEL) )  
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC2, 11);     // write Configuration  433.6 MHz  Channel 1
      gpio_put(CSN905,1);

      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR2, 5);
      gpio_put(CSN905,1);
   }
   else 
   {
      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, WC1, 11);   // write Configuration  433.4 MHz  Channel 2
      gpio_put(CSN905,1);
     
      sleep_us(1000);

      gpio_put(CSN905,0);
      spi_write_blocking(spi_default, TXADR1, 5);
      gpio_put(CSN905,1);
   }

   sleep_ms(100);

// Reading the Configuration
   spi_out_buffer[0] = 0x10;   // command for reading Configuration

   gpio_put(CSN905,0);   
   spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, 11);
   gpio_put(CSN905,1);   

// Display the Configuration data
   lcd_pos = 0x80;
   for(n=1; n<11; n++)
   {
      uint8_hex_lcd(spi_in_buffer[n],lcd_pos);
      lcd_pos = lcd_pos + 3;

      if(n == 5) {lcd_pos = 0xC0;}
   }
   
   sleep_ms(2000);

   gpio_put(TXEN,0);     // RX Mode
   sleep_us(20);
   gpio_put(TRX_CE,1);   

// Do a dummy read to clear things so it will work properly from power-up
   gpio_put(TXEN,0);
   sleep_us(20);
   gpio_put(TRX_CE,0);   // Read Data

// Initialize output data buffer; first byte is Read RX Payload command
   spi_out_buffer[0] = 0x24;

// Get the RX data
   gpio_put(CSN905,0);   
// Write the output buffer to MOSI, and at the same time read from MISO.
   spi_write_read_blocking(spi_default, spi_out_buffer, spi_in_buffer, BUF_LEN);
   gpio_put(CSN905,1); 

   gpio_put(TXEN,0);   // RX Mode
   sleep_us(20);
   gpio_put(TRX_CE,1);  
 //-----------
   SetCmdMode();    // set for commands
   lcd_byte(0x01);  // clear screen
}
//==========================================
void my_itoa(unsigned int bin,char *str)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//===============================================
// unsigned int to ASCII string, remove leading zeros
void my_itoa2(unsigned int bin,char *str)  
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Make the string
// Display it
   if(char1 != 0x30)
   { 
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = 0;
   }
   else
   { 
      if(char2 != 0x30)
      { 
         str[0] = char2;
         str[1] = char3;
         str[2] = char4;
         str[3] = 0;
      }
      else
      {   
         if(char3 != 0x30)
         { 
            str[0] = char3;
            str[1] = char4;
            str[2] = 0;
         }
         else
         {
            str[0] = char4;
            str[1] = 0;
         }
      }
   }
}
//===============================================
// Fills a string with two ASCII numbers from two BCD numbers; ends with NULL if desired
void line_fill(unsigned char hex, char *ptr, unsigned char index, unsigned char end)
{
   unsigned char digit1, digit2;

   digit1 =  hex>>4;
   digit2 = hex & 0x0F;

   if(digit1<10)
      {digit1 = digit1 + 0x30;}
   else 
      { digit1 = digit1 + 0x37;}

   if(digit2<10)  
      {digit2 = digit2 + 0x30;}
   else 
      { digit2 = digit2 + 0x37;}

   ptr[index++] = digit1;
   ptr[index] = digit2;
   if(end) { ptr[++index] = 0; }  // NULL
}
//========================================================
// Send out time and an increasing number
void test_uart()
{
   ts_number = 0;
   uart_puts(UART_ID,"UART Time Stamp Test\n");

   while(1)
   {

      log1_time_uart();
      strcat(line,"\n");

      uart_puts(UART_ID, line);   // put it out now
      ts_number++;                // for testing

      sleep_ms(1000);
   }
}
//===============================================
// Just echo the keyboard; does not come back
void keyboard_echo()
{
   uint16_t n;

// make sure uart FIFO is empty, disable FIFO for this
   uart_set_fifo_enabled(UART_ID, false);
   for(n=0; n<32; n++)   
   {
       // keep going until FIFO is empty (0 bytes in FIFO)
       if( uart_is_readable(UART_ID) )   { uart_char = uart_getc(UART_ID); }
       else {break;}
   }
   //uart_set_fifo_enabled(UART_ID, true);  // Keep it off

   uart_puts(UART_ID, "Echo the Keyboard\n\n");

   while(1)
   {
      if( uart_is_readable(UART_ID) )
      {
//       uart_char = uart_getc(UART_ID & (~0x20);  // Read uart FIFO (case insensitive)
         uart_char = uart_getc(UART_ID);  // Read uart FIFO (case sensitive)
         uart_putc(UART_ID, uart_char);
      }
   }
}
//========================================================
void LED_break()
{
   while(1)  
   {
      gpio_put(LED1,1);
      sleep_ms(250); 
      gpio_put(LED1,0);
      sleep_ms(250); 
   }
}
//=====================================================
void log1_time_uart()
{
   unsigned int m, n;

// ts_number is the log number starting from Reset
   my_itoa(ts_number,nstr);  // ends with NULL
   strcpy(line, nstr);
   strcat(line,"    ");   

   i2c_get_time();

   n = time_buffer[DAY] - 1;   // use it as an index
   for(m=0; m<3; m++)
   {
      nstr[m] = day_array[n][m];
   }
   nstr[m] = 0;
   strcat(line,nstr);
   strcat(line,"    ");   

   line_fill(time_buffer[MONTH], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[DATE], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[YEAR], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(time_buffer[HOURS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[MINS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[SECS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

//   strcat(line,"\n");
}
//==============================================
void log2_time_uart()   // without tx_number
{
//   strcpy(line,"            ");   

   i2c_get_time();

   line_fill(time_buffer[MONTH], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[DATE], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"/");

   line_fill(time_buffer[YEAR], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

   line_fill(time_buffer[HOURS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[MINS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,":");

   line_fill(time_buffer[SECS], nstr, 0, 1);
   strcat(line,nstr);
   strcat(line,"    ");

//   strcat(line,"\n");
}
//===============================================
// unsigned char to BCD to string to uart
void uint8_bcd_str(unsigned char bin, char *str)   //unsigned char to BCD string    
{
// First Convert unsigned hex byte to BCD 
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3;
   unsigned char hcarry;

//bin = 0xA2;
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Put it in the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = 0;      // NULL
}
//=================================================
// signed char to BCD to string to uart
void sint8_bcd_str(int8_t bin, char *str)   //unsigned char to BCD string    
{
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3, sflag;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1;
   }
   else {sflag = 0;}

// Convert unsigned hex byte to BCD 
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Put it in the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = 0;      // NULL

   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = 0;      // NULL
   }
}
//=================================================
// unsigned int to BCD to string to uart
void uint16_bcd_str(unsigned int bin, char *str)  //unsigned int to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = char5;
   str[5] = 0;  // NULL
}
//=================================================
// signed int to BCD to string to uart
void sint16_bcd_str(int bin, char *str)  //unsigned int to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5, sflag;
   int i;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1; 
   }
   else {sflag = 0; }

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Make the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = char4;
      str[5] = char5;
      str[6] = 0;      // NULL
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = char5;
      str[5] = 0;      // NULL
   }
}
//===============================================
void uint32_bcd_str(uint32_t bin, char *str)  //uint32_t to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Make the string
   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = char5;
   str[5] = char6;
   str[6] = char7;
   str[7] = char8;
   str[8] = char9;
   str[9] = char10;
   str[10] = 0;  // NULL
}
//===============================================
void sint32_bcd_str(int32_t bin, char *str)    // int32_t to BCD string
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10, sflag;
   int i;
   unsigned char hcarry;

   if(bin <0)
   {
      bin = -bin;
      sflag = 1;
   }
   else { sflag = 0; }

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Make the string
   if(sflag)
   {
      str[0] = '-';
      str[1] = char1;
      str[2] = char2;
      str[3] = char3;
      str[4] = char4;
      str[5] = char5;
      str[6] = char6;
      str[7] = char7;
      str[8] = char8;
      str[9] = char9;
      str[10] = char10;
      str[11] = 0;  // NULL
   }
   else
   {
      str[0] = char1;
      str[1] = char2;
      str[2] = char3;
      str[3] = char4;
      str[4] = char5;
      str[5] = char6;
      str[6] = char7;
      str[7] = char8;
      str[8] = char9;
      str[9] = char10;
      str[10] = 0;  // NULL
   }
}
//===============================================
// unsigned 8 bits to hex to uart
void uint8_hex_str(uint8_t bin, char *str)   //unsigned char to hex string    
{
   unsigned char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Make string
   str[0] = char1;
   str[1] = char2;
   str[2] = 0;
}
//==========================================================
// unsigned 16 bits to hex to uart
void uint16_hex_str(unsigned int bin, char *str)  //unsigned int to hex string
{
   unsigned char1, char2, char3, char4, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char4 = temp + 0x30;}
   else
     {char4 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char3 = temp + 0x30;}
   else
     {char3 = temp + 0x37;}

   temp = bin>>8 & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>12 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}

   str[0] = char1;
   str[1] = char2;
   str[2] = char3;
   str[3] = char4;
   str[4] = 0;  // NULL
}
//============================================
// unsigned 32 bits to hex to uart
void uint32_hex_str(unsigned long bin, char *str)  //unsigned long to hex string
// long int hex to string, space between ints
{
// Convert Binary to BCD
   unsigned int m, n;
   char temp[10];

   m = 7;
   for(n=0; n<8; n++)
   {
      temp[m] = bin & 0x0F;
      m--;
      bin = bin>>4;
   }
   asm(" nop");    // for emulator

   // Convert to ascii
   for(n=0; n<8; n++)
   {
     if(temp[n]<10)  { temp[n] = temp[n] + 0x30; }
     else            { temp[n] = temp[n] + 0x37; }
   }
  
// Format the string
   for(n=0; n<4; n++)  { str[n] = temp[n]; }
   str[4] = ' ';

   for(n=4; n<8; n++)  { str[n+1] = temp[n]; }
   str[9] = 0x00;  // NULL   Now it is a string

   asm(" nop");    // for emulator
}
//========================================================
// 8 bits hex to lcd
void uint8_hex_lcd(uint8_t bin, uint8_t pos)        //unsigned char to hex to lcd   
{
   uint8_t char1, char2, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}
    
// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
}
//==========================================================
// 16 bits hex to lcd
void uint16_hex_lcd(uint16_t bin, uint8_t pos)   //unsigned int to BCD to lcd
{
   unsigned char1, char2, char3, char4, temp;

// Convert to ascii
   temp = bin & 0x0F;
   if(temp<10)
     {char4 = temp + 0x30;}
   else
     {char4 = temp + 0x37;}

   temp = bin>>4 & 0x0F;
   if(temp<10)
     {char3 = temp + 0x30;}
   else
     {char3 = temp + 0x37;}

   temp = bin>>8 & 0x0F;
   if(temp<10)
     {char2 = temp + 0x30;}
   else
     {char2 = temp + 0x37;}

   temp = bin>>12 & 0x0F;
   if(temp<10)
     {char1 = temp + 0x30;}
   else
     {char1 = temp + 0x37;}

// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);
}
//===============================================
// 32 bits hex to lcd
void uint32_hex_lcd(uint32_t bin, uint8_t pos)        //unsigned long to hex to lcd
{
   unsigned int m, n;
   char temp[10];
 
 // Convert Binary to BCD
   m = 7;
   for(n=0; n<8; n++)
   {
      temp[m] = bin & 0x0F;
      m--;
      bin = bin>>4;
   }
//   asm(" nop");    // for emulator

// Convert to ascii
   for(n=0; n<8; n++)
   {
     if(temp[n]<10)  { temp[n] = temp[n] + 0x30; }
     else            { temp[n] = temp[n] + 0x37; }
   }
  
// Display it
   lcd_wr_instr(pos);
   for(n=0; n<4; n++)  { lcd_wr_data(temp[n]); }

   lcd_wr_data(' ');

   for(n=4; n<8; n++)  { lcd_wr_data(temp[n]); }
   asm(" nop");    // for emulator
}
//========================================================
// unsigned 8 bits to BCD to lcd
void uint8_bcd_lcd(uint8_t bin, uint8_t pos)   //unsigned char to BCD to lcd
{
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3;
   unsigned char hcarry;

// Convert unsigned hex byte to BCD 
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
}
//=================================================
// signed 8 bits to BCD to lcd
void sint8_bcd_lcd(int8_t bin, uint8_t pos)   //nsigned char to BCD to lcd
{
   int i;  
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
 
   unsigned char char1, char2, char3, sflag;
   unsigned char hcarry;

   if(bin<0)  
   {
      bin = -bin;
      sflag = 1;
   }
   else {sflag = 0;} 

// Convert unsigned hex byte to BCD 
   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

      if(bcd2 >9)
      {
          bcd2 = bcd2 - 10;
//          hcarry = 1;
      }
//      else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd2 + 0x30;
   char2 = bcd1 + 0x30;
   char3 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);
   if(sflag)  { lcd_wr_data('-'); }
   if(char1 != '0')  { lcd_wr_data(char1); }
   lcd_wr_data(char2);
   lcd_wr_data(char3);
}
//=================================================
// unsigned 16 bits to BCD to lcd
void uint16_bcd_lcd(uint16_t bin, uint8_t pos)   //unsigned int to BCD yo lcd
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);
   lcd_wr_data(char5);
}
//=================================================
// unsigned 16 bits to 4 BCD digits to lcd with some leading zero suppression
void uint16_bcd4_lcd(uint16_t bin, uint8_t pos)   //unsigned int to BCD yo lcd
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char char1, char2, char3, char4;
   int i;
   unsigned char hcarry;

 // Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd3 + 0x30;
   char2 = bcd2 + 0x30;
   char3 = bcd1 + 0x30;
   char4 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(char1 == 0x30)
    { lcd_wr_data(' '); }
   else
    { 
       lcd_wr_data(char1);
       lcd_wr_data(char2);
       lcd_wr_data(char3);
       lcd_wr_data(char4);
       return;
    }

   if(char2 == 0x30)
    { lcd_wr_data(' '); }
   else
    { lcd_wr_data(char2);}

   lcd_wr_data(char3);
   lcd_wr_data(char4);
}
//===============================================
// signed 16 bits to BCD to lcd
void sint16_bcd_lcd(int16_t bin, uint8_t pos)   //unsigned int to BCD yo lcd
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char char1, char2, char3, char4, char5, sflag;
   int i;
   unsigned char hcarry;

   if(bin<0)  
   {
      bin = -bin;
      sflag = 1;
   }
   else {sflag = 0;} 

// Convert Binary to BCD
   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
       hcarry = 1;
     }
         else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
//       hcarry = 1;
     }
//         else {hcarry = 0;}

   }

// Convert to ascii
   char1 = bcd4 + 0x30;
   char2 = bcd3 + 0x30;
   char3 = bcd2 + 0x30;
   char4 = bcd1 + 0x30;
   char5 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);
   if(sflag)  { lcd_wr_data('-'); }
//   if(char1 != '0')  
   lcd_wr_data(char1);
   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);
   lcd_wr_data(char5);
}
//===============================================
// unsigned 32 bits to BCD to lcd
void uint32_bcd_lcd(uint32_t bin, uint8_t pos)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10;
   int i;
   unsigned char hcarry;

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   lcd_wr_data(char1);

   lcd_wr_data(',');

   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);

   lcd_wr_data(',');
   
   lcd_wr_data(char5);
   lcd_wr_data(char6);
   lcd_wr_data(char7);
   
   lcd_wr_data(',');
   
   lcd_wr_data(char8);
   lcd_wr_data(char9);
   lcd_wr_data(char10);
}
//===============================================
// signed 32 bits to BCD to lcd
void sint32_bcd_lcd(int32_t bin, uint8_t pos)
{
   unsigned char bcd0 = 0;
   unsigned char bcd1 = 0;
   unsigned char bcd2 = 0;
   unsigned char bcd3 = 0;
   unsigned char bcd4 = 0;
   unsigned char bcd5 = 0;
   unsigned char bcd6 = 0;
   unsigned char bcd7 = 0;
   unsigned char bcd8 = 0;
   unsigned char bcd9 = 0;
   unsigned char char1, char2, char3, char4, char5, char6, char7, char8, char9, char10, sflag;
   int i;
   unsigned char hcarry;

   if(bin<0)
   {
      bin = -bin;
      sflag = 1;
   }
   else {sflag = 0;}

// Convert Binary to BCD
   for(i=0; i<32; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80000000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD4
     bcd4 = bcd4<<1;
     bcd4 = bcd4 + hcarry;

     if(bcd4 >9)
     {
         bcd4 = bcd4 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD5
     bcd5 = bcd5<<1;
     bcd5 = bcd5 + hcarry;

     if(bcd5 >9)
     {
         bcd5 = bcd5 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD6
     bcd6 = bcd6<<1;
     bcd6 = bcd6 + hcarry;

     if(bcd6 >9)
     {
         bcd6 = bcd6 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD7
     bcd7 = bcd7<<1;
     bcd7 = bcd7 + hcarry;

     if(bcd7 >9)
     {
         bcd7 = bcd7 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD8
     bcd8 = bcd8<<1;
     bcd8 = bcd8 + hcarry;

     if(bcd8 >9)
     {
         bcd8 = bcd8 - 10;
         hcarry = 1;
     }
     else {hcarry = 0;}

//---------- BCD9
     bcd9 = bcd9<<1;
     bcd9 = bcd9 + hcarry;

     if(bcd9 >9)
     {
         bcd9 = bcd9 - 10;
//         hcarry = 1;
     }
//   else {hcarry = 0;}
   }  // end for

// Convert to ascii
   char1 = bcd9 + 0x30;
   char2 = bcd8 + 0x30;
   char3 = bcd7 + 0x30;
   char4 = bcd6 + 0x30;
   char5 = bcd5 + 0x30;
   char6 = bcd4 + 0x30;
   char7 = bcd3 + 0x30;
   char8 = bcd2 + 0x30;
   char9 = bcd1 + 0x30;
   char10 = bcd0 + 0x30;

// Display it
   lcd_wr_instr(pos);

   if(sflag)  { lcd_wr_data('-'); }   // minus sign?
   lcd_wr_data(char1);

   lcd_wr_data(',');

   lcd_wr_data(char2);
   lcd_wr_data(char3);
   lcd_wr_data(char4);

   lcd_wr_data(',');
   
   lcd_wr_data(char5);
   lcd_wr_data(char6);
   lcd_wr_data(char7);
   
   lcd_wr_data(',');
   
   lcd_wr_data(char8);
   lcd_wr_data(char9);
   lcd_wr_data(char10);
}
//=================================================
// dispay binary
void bin12_lcd( unsigned int value, unsigned char pos)
{
   unsigned char n;
   unsigned int mask;

// Display it
   lcd_wr_instr(pos);

   mask = 0x0800;
//   value = 0x055A;   // test

   for(n=0; n<12; n++)
   {
      if( (value & mask) == 0)
        { lcd_wr_data('0'); }
      else
        { lcd_wr_data('1'); }

      if( (n==3) | (n==7) ) { lcd_wr_data(' ');}  
      mask = mask>>1;
   }
}
//=======================================
// display binary
void bin12_str( unsigned int value, char *str)
{
   unsigned char m, n;
   unsigned int mask;

// Make string
   mask = 0x0800;
   m = 0;

   for(n=0; n<12; n++)
   {
      if( (value & mask) == 0) { str[m++] = '0'; }
      else { str[m++] = '1'; }

      if( (n==3) | (n==7) ) { nstr[m++] = ' ';}  

      mask = mask>>1;
   }

   nstr[m] = 0;  // NULL
}
//======================================================================
