// lcd.c  lcd functions
#include <stdio.h>
#include <string.h>
#include "pico/stdlib.h"
#include "hardware/i2c.h"
#include "pico/binary_info.h"

#define LCD_E 10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

void SetChrMode(void);
void SetCmdMode(void);
void lcd_init(void);
void domsg(const struct messages *);
void lcd_wr_data(unsigned char);
void lcd_wr_instr(unsigned char);
void lcd_text(char *);
void lcd_byte(char);
void lcd_wr_time(unsigned int, unsigned int);
void clear_lcd(unsigned char);
//================================================================
void lcd_wr_time(unsigned int pos, unsigned int bcd)
{
   unsigned char char1, char2;

// Convert to ascii
   char1 = (bcd>>4) + 0x30;
   char2 = (bcd & 0x0F) + 0x30;

   lcd_wr_instr(pos);
   lcd_wr_data(char1);
   lcd_wr_data(char2);
}
//====================================
void pulseEnable ()
{
   gpio_put(LCD_E,1);
   sleep_us(1000);   // 1000 microseconds

   gpio_put(LCD_E,0);
   sleep_us(1000);
}
//==========================================
//  send a byte to the lcd in two nibbles
//  before calling use SetChrMode or SetCmdMode to determine whether to send character or command
void lcd_byte(char bits)
{
   if(bits & 0x10)  {gpio_put(LCD_D4,1);}
   else {gpio_put(LCD_D4,0);}

   if(bits & 0x20)  {gpio_put(LCD_D5,1);}
   else {gpio_put(LCD_D5,0);}

   if(bits & 0x40)  {gpio_put(LCD_D6,1);}
   else {gpio_put(LCD_D6,0);}

   if(bits & 0x80)  {gpio_put(LCD_D7,1);}
   else {gpio_put(LCD_D7,0);}

   pulseEnable();

   if(bits & 0x01)  {gpio_put(LCD_D4,1);}
   else {gpio_put(LCD_D4,0);}

   if(bits & 0x02)  {gpio_put(LCD_D5,1);}
   else {gpio_put(LCD_D5,0);}

   if(bits & 0x04)  {gpio_put(LCD_D6,1);}
   else {gpio_put(LCD_D6,0);}

   if(bits & 0x08)  {gpio_put(LCD_D7,1);}
   else {gpio_put(LCD_D7,0);}

   pulseEnable();      	
}
//--------------------------------------------
void SetCmdMode() 
{
  gpio_put(LCD_RS,0);   // command mode
}
//--------------------------------------------
void SetChrMode() 
{
   gpio_put(LCD_RS,1);  // data mode
}
//-------------------------------------------
void lcd_text(char *s)
{
  while(*s) 
	lcd_byte(*s++);
 }
//--------------------------------------------
void lcd_init()
{
// set up pi pins for output
    gpio_init(LCD_E);
    gpio_set_dir(LCD_E, GPIO_OUT);
    gpio_put(LCD_E, 0);

    gpio_init(LCD_RS);
    gpio_set_dir(LCD_RS, GPIO_OUT);
    gpio_put(LCD_RS, 0);

    gpio_init(LCD_D4);
    gpio_set_dir(LCD_D4, GPIO_OUT);
    gpio_put(LCD_D4, 0);

    gpio_init(LCD_D5);
    gpio_set_dir(LCD_D5, GPIO_OUT);
    gpio_put(LCD_D5, 0);

    gpio_init(LCD_D6);
    gpio_set_dir(LCD_D6, GPIO_OUT);
    gpio_put(LCD_D6, 0);

    gpio_init(LCD_D7);
    gpio_set_dir(LCD_D7, GPIO_OUT);
    gpio_put(LCD_D7, 0);

   // initialise LCD
   SetCmdMode(); // set for commands
   lcd_byte(0x33); // full init 
   lcd_byte(0x32); // 4 bit mode
   lcd_byte(0x28); // 2 line mode
   lcd_byte(0x0C); // display on, cursor off, blink off
   lcd_byte(0x01);  // clear screen

// Function Set:  0  0  1  DL    N  F  x  x
//                0  0  1  0     1  0  0  0

   sleep_ms(3);
}
//=================================================
// Do LCD Message
void domsg(const struct messages *sptr)
{
    unsigned char m,n;
    unsigned char data;

    m = sptr->pos;  // position
    lcd_wr_instr(m);

    m = sptr->numchars;  // numbers of chars

    for(n=0; n<m; n++)
    {
        data = sptr->msg[n];
        lcd_wr_data(data);
    }
}
//==========================================
// LCD - Write 8-bit instruction
// LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_instr(unsigned char data)
{
//   sleep_us(60);
   gpio_put(LCD_RS,0);   // RS = 0; command mode
//   sleep_us(60);
   lcd_byte(data);
   gpio_put(LCD_RS,1);   // RS = 0; data mode
//   sleep_us(60);
}
//======================
// LCD - Write 8-bit data
// LCD RS (Register Select; RS low = Instruction, RS High = Data)
void lcd_wr_data(unsigned char data)
{
    gpio_put(LCD_RS,1);   // data mode, RS = 1
    lcd_byte(data);
}
//=================================================
void clear_lcd(unsigned char pos)
{
    unsigned int n;

    lcd_wr_instr(pos);

    for(n=16; n>0; --n)
    { lcd_wr_data(' ');}
}
//=========================================================
