// main.h

const char version[64] = "\nJed Margolin's ADC Test   7/8/2024";  // for uart

// GPIO Pins
#define LCD_E  10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

#define PB1 14
#define PB2 13
#define PB3 12
#define SW  15

#define CHANNEL 3   // RF Channel
#define LED1 2      // Green
#define LED2 27     // Red

#define TXEN 21    // nrf905 Transmit Enable
#define TRX_CE 20  // enable nrf905 chip for transmit/receive
#define AM 28      // Address Match
#define DR 22      // Data Ready
#define CSN905 17  // nrf906 CSN - pin 22
#define BUF_LEN  33

#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

#define ADC0_PIN 26

#define REGMODE 23    // voltage regulator mode  1 = quiet mode
//--------------------------------
// pushbutton switches
extern struct myswitch switch1;   // also defined and used in clock.h
extern struct myswitch switch2;
extern struct myswitch switch3;

extern void do_switch(unsigned char, struct myswitch *); 

// for nrf905
// The first byte is the instruction and the address start for nrf905, and example is WC1[0]
unsigned char WC1[11] = { 0x00, 0x70, 0x0C, 0x44, 0x20, 0x20, 0x61, 0x62, 0x63, 0x64, 0xDB };  // Test 433.6 MHz
unsigned char TXADR1[5] = { 0x22, 0x61, 0x62, 0x63, 0x64 };  // TXADR[0] is the instruction

unsigned char WC2[11] = { 0x00, 0x6E, 0x0C, 0x44, 0x20, 0x20, 0x54, 0x53, 0x52, 0x51, 0xDB };  // Garage 433.4 MHz
unsigned char TXADR2[5] = { 0x22, 0x54, 0x53, 0x52, 0x51 };  // TXADR[0] is the instruction

unsigned char spi_out_buffer[BUF_LEN];   // out_buffer[0] is the instruction
unsigned char spi_in_buffer[BUF_LEN];    // in_buffer[0] is the status byte

// Messages - structure is duplicated in clock.h
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

const struct messages tmsg1 = {0x80, 14, "Jed Margolin's"};
const struct messages tmsg2 = {0xC1, 12,  "ADC/RTC Test"};
const struct messages tmsg3 = {0x83, 8,  "(C) 2024"};
const struct messages tmsg4 = {0x8C, 4, "12-B"};
const struct messages tmsg5 = {0xCC, 4, "10-B"};
const struct messages tmsg6 = {0x83, 9, "ADC0 Test"};
const struct messages tmsg7 = {0x80, 3, "Hi:"};
const struct messages tmsg8 = {0x89, 3, "Lo:"};
const struct messages tmsg9 = {0xC0, 2, "D:"};
const struct messages tmsg10 = {0xC8, 4, "AVG:"};
const struct messages tmsg11 = {0x83, 8, "7/8/2024"};
const struct messages tmsg12 = {0x84, 6, "nrf905"};
//=================================================================
extern void SetChrMode(void);
extern void SetCmdMode(void);
extern void lcd_init(void);
extern void domsg(const struct messages *);
extern void lcd_wr_data(unsigned char);
extern void lcd_wr_instr(unsigned char);
extern void lcd_text(char *);
extern void lcd_byte(char);
extern void lcd_wr_time(unsigned int, unsigned int);

//   i2c_write_blocking(i2c_default, DSADR, out_buffer, nbytes_out, false);
uint8_t out_buffer[10];   // first byte is register address

extern unsigned char nflag;
struct repeating_timer timer;

void do_adc_tests(void);
extern void do_clock(void);
//=======================================================
unsigned char mode;   // 1 = ADC Tests  0 = Clock
unsigned char irq_flag, secs_flag;

unsigned int save_count, irq_count, ts_number;
unsigned char nflag;

char nstr[128];
char line[256];

unsigned int adcval_max, adcval_min, adcval_diff;
unsigned int adc_buffer[1024];
unsigned long adc_sum;
unsigned int adc_avg, adc_high, adc_low, adc_diff;
uint16_t frame, frame_flag;
int16_t test_mode;
//=================================================================
void my_itoa(unsigned int, char *);
void my_itoa2(unsigned int, char *);
void bin12_str(unsigned int, char *);

void line_fill(unsigned char, char *, unsigned char, unsigned char);

void test_uart(void);
void log1_time_uart(void);
void log2_time_uart(void);
void my_hextoa2(unsigned char, char *);
void my_hextoa8(uint32_t, char *); 
void LED_break(void);
void nrf905_init(void);

void do_adc_test1(uint16_t, uint16_t);
void do_adc_test2(void);
void bin12_lcd(unsigned int, unsigned char);

extern void clear_lcd(unsigned char);

// From clock.c
#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6

extern const char day_array[7][3];
extern const char *month_name[12];
extern void i2c_get_time(void);
extern unsigned char time_buffer[];
extern const struct messages msg1, msg2, msg3,msg15;
//===============================================================
unsigned char uart_char;
void keyboard_echo(void);
void do_uart_RX(void);
uint32_t ints;
//================================================
void do_time(void);
void results_uart(uint16_t);

void uint8_bcd_str(unsigned char, char *);   // unsigned char to BCD string    
void uint16_bcd_str(unsigned int, char *);   // unsigned int to BCD string
void uint32_bcd_str(uint32_t, char *);       // unsigned long to BCD string

void sint8_bcd_str(int8_t, char *);          // signed char to BCD string    
void sint16_bcd_str(int, char *);            // signed int to BCD string
void sint32_bcd_str(int32_t, char *);        // signed long to BCD string

void uint8_hex_str(unsigned char, char *);   // unsigned char to hex string    
void uint16_hex_str(unsigned int, char *);   // unsigned int to hex string
void uint32_hex_str(unsigned long, char *);  // unsigned long to hex string

void uint8_bcd_lcd(uint8_t, uint8_t);        // unsigned char to BCD to lcd    
void uint16_bcd_lcd(uint16_t, uint8_t);      // unsigned int to BCD to lcd
void uint16_bcd4_lcd(uint16_t, uint8_t);     // unsigned int to BCD to lcd with some leading zero suppression
void uint32_bcd_lcd(uint32_t bin, uint8_t pos);  // unsigned long to BCD lcd

void sint8_bcd_lcd(int8_t bin, uint8_t pos);     // signed char to BCD to lcd    
void sint16_bcd_lcd(int16_t, uint8_t);           // signed int to BCD to lcd
void sint32_bcd_lcd(int32_t bin, uint8_t pos);   // signed long to BCD to lcd

void uint8_hex_lcd(uint8_t, uint8_t);    // unsigned char to hex to lcd    
void uint16_hex_lcd(uint16_t, uint8_t);  // unsigned int to hex to lcd
void uint32_hex_lcd(uint32_t, uint8_t);  // unsigned long to hex to lcd

void wait_pb1(void);

// For reading, debouncing, and processing the switches
struct myswitch
{
    unsigned char debounce;     // debounce byte
    unsigned char state;        // debounced state of the switch
    unsigned char old;          // old state
    unsigned char press;        // switch has been pressed
    unsigned char release;      // switch has been released
};
// Declare and initialize the switches
extern struct myswitch switch1;
extern struct myswitch switch2;
extern struct myswitch switch3;

/*  RPI Pico sizes
Name 	            C++ 	Bytes 	Range

bool 		                    1 	true or false
signed char 	    int8_t 	    1 	-128 to 127
unsigned char 	    uint8_t 	1 	0 to 255
short int 	        int16_t 	2 	-32768 to 32767
unsigned short int 	uint16_t 	2 	0 to 65535
int 	            int32_t 	4 	-2147483648 to 2147483647
unsigned int 	    uint32_t 	4 	0 to 4294967295
long int 	        int32_t 	4 	-2147483648 to 2147483647
unsigned long int 	uint32_t 	4 	0 to 4294967295
long long 	        int64_t 	8 	−9,223,372,036,854,775,808 to 9,223,372,036,854,775,807
unsigned long long 	uint64_t 	8 	0 to 18,446,744,073,709,551,615
float 		                    4 	+/- 3.4e +/- 38 (~7 digits)
double 		                    8 	+/- 1.7e +/- 308 (~15 digits)
long double 		            8 	+/- 1.7e +/- 308 (~15 digits)
wchar_t 		            2 or 4  1 wide character
*/
